@@ -46,12 +46,13 @@ TargetNVC0::TargetNVC0(unsigned int card) : Target(false, card >= 0xe4)
 void
 TargetNVC0::getBuiltinCode(const uint32_t **code, uint32_t *size) const
 {
-   switch (chipset & 0xf0) {
+   switch (chipset & ~0xf) {
    case 0xe0:
       *code = (const uint32_t *)&nve4_builtin_code[0];
       *size = sizeof(nve4_builtin_code);
       break;
    case 0xf0:
+   case 0x100:
       *code = (const uint32_t *)&nvf0_builtin_code[0];
       *size = sizeof(nvf0_builtin_code);
       break;
@@ -67,9 +68,12 @@ TargetNVC0::getBuiltinOffset(int builtin) const
 {
    assert(builtin < NVC0_BUILTIN_COUNT);
 
-   switch (chipset & 0xf0) {
-   case 0xe0: return nve4_builtin_offsets[builtin];
-   case 0xf0: return nvf0_builtin_offsets[builtin];
+   switch (chipset & ~0xf) {
+   case 0xe0:
+      return nve4_builtin_offsets[builtin];
+   case 0xf0:
+   case 0x100:
+      return nvf0_builtin_offsets[builtin];
    default:
       return nvc0_builtin_offsets[builtin];
    }
@@ -120,7 +124,10 @@ static const struct opProperties _initProps[] =
    { OP_DFDX,   0x1, 0x0, 0x0, 0x0, 0x0, 0x0 },
    { OP_DFDY,   0x1, 0x0, 0x0, 0x0, 0x0, 0x0 },
    { OP_CALL,   0x0, 0x0, 0x0, 0x0, 0x1, 0x0 },
-   { OP_INSBF,  0x0, 0x0, 0x0, 0x0, 0x0, 0x4 },
+   { OP_POPCNT, 0x0, 0x0, 0x3, 0x0, 0x2, 0x2 },
+   { OP_INSBF,  0x0, 0x0, 0x0, 0x0, 0x6, 0x2 },
+   { OP_EXTBF,  0x0, 0x0, 0x0, 0x0, 0x2, 0x2 },
+   { OP_BFIND,  0x0, 0x0, 0x1, 0x0, 0x1, 0x1 },
    { OP_PERMT,  0x0, 0x0, 0x0, 0x0, 0x6, 0x2 },
    { OP_SET_AND, 0x3, 0x3, 0x0, 0x0, 0x2, 0x2 },
    { OP_SET_OR, 0x3, 0x3, 0x0, 0x0, 0x2, 0x2 },
@@ -278,6 +285,9 @@ TargetNVC0::getSVAddress(DataFile shaderFile, const Symbol *sym) const
    case SV_NTID:           return kepler ? (0x00 + idx * 4) : ~0;
    case SV_NCTAID:         return kepler ? (0x0c + idx * 4) : ~0;
    case SV_GRIDID:         return kepler ? 0x18 : ~0;
+   case SV_SAMPLE_INDEX:   return 0;
+   case SV_SAMPLE_POS:     return 0;
+   case SV_SAMPLE_MASK:    return 0;
    default:
       return 0xffffffff;
    }
@@ -357,13 +367,6 @@ TargetNVC0::isAccessSupported(DataFile file, DataType ty) const
       return typeSizeof(ty) <= 8;
    if (ty == TYPE_B96)
       return false;
-   if (getChipset() >= 0xf0) {
-      // XXX: find wide vfetch/export
-      if (ty == TYPE_B128)
-         return false;
-      if (ty == TYPE_U64)
-         return false;
-   }
    return true;
 }
 
@@ -393,6 +396,8 @@ TargetNVC0::isModSupported(const Instruction *insn, int s, Modifier mod) const
       case OP_AND:
       case OP_OR:
       case OP_XOR:
+      case OP_POPCNT:
+      case OP_BFIND:
          break;
       case OP_SET:
          if (insn->sType != TYPE_F32)
