@@ -25,10 +25,9 @@
  *
  */
 
-#include "brw_fs.h"
 #include "brw_cfg.h"
 
-/** @file brw_cfg_t.cpp
+/** @file brw_cfg.cpp
  *
  * Walks the shader instructions generated and creates a set of basic
  * blocks with successor/predecessor edges connecting them.
@@ -52,48 +51,52 @@ bblock_t::bblock_t() :
 
    parents.make_empty();
    children.make_empty();
+
+   if_inst = NULL;
+   else_inst = NULL;
+   endif_inst = NULL;
 }
 
 void
 bblock_t::add_successor(void *mem_ctx, bblock_t *successor)
 {
-   successor->parents.push_tail(this->make_list(mem_ctx));
-   children.push_tail(successor->make_list(mem_ctx));
-}
-
-bblock_link *
-bblock_t::make_list(void *mem_ctx)
-{
-   return new(mem_ctx) bblock_link(this);
-}
-
-cfg_t::cfg_t(backend_visitor *v)
-{
-   create(v->mem_ctx, &v->instructions);
-}
-
-cfg_t::cfg_t(void *mem_ctx, exec_list *instructions)
-{
-   create(mem_ctx, instructions);
+   successor->parents.push_tail(new(mem_ctx) bblock_link(this));
+   children.push_tail(new(mem_ctx) bblock_link(successor));
 }
 
 void
-cfg_t::create(void *parent_mem_ctx, exec_list *instructions)
+bblock_t::dump(backend_visitor *v)
+{
+   int ip = this->start_ip;
+   for (backend_instruction *inst = (backend_instruction *)this->start;
+	inst != this->end->next;
+	inst = (backend_instruction *) inst->next) {
+      fprintf(stderr, "%5d: ", ip);
+      v->dump_instruction(inst);
+      ip++;
+   }
+}
+
+cfg_t::cfg_t(exec_list *instructions)
 {
    mem_ctx = ralloc_context(NULL);
    block_list.make_empty();
    blocks = NULL;
    num_blocks = 0;
-   ip = 0;
-   cur = NULL;
+
+   bblock_t *cur = NULL;
+   int ip = 0;
 
    bblock_t *entry = new_block();
-   bblock_t *cur_if = NULL, *cur_else = NULL, *cur_endif = NULL;
-   bblock_t *cur_do = NULL, *cur_while = NULL;
-   exec_list if_stack, else_stack, endif_stack, do_stack, while_stack;
+   bblock_t *cur_if = NULL;    /**< BB ending with IF. */
+   bblock_t *cur_else = NULL;  /**< BB ending with ELSE. */
+   bblock_t *cur_endif = NULL; /**< BB starting with ENDIF. */
+   bblock_t *cur_do = NULL;    /**< BB ending with DO. */
+   bblock_t *cur_while = NULL; /**< BB immediately following WHILE. */
+   exec_list if_stack, else_stack, do_stack, while_stack;
    bblock_t *next;
 
-   set_next_block(entry);
+   set_next_block(&cur, entry, ip);
 
    entry->start = (backend_instruction *) instructions->get_head();
 
@@ -110,16 +113,12 @@ cfg_t::create(void *parent_mem_ctx, exec_list *instructions)
 	 /* Push our information onto a stack so we can recover from
 	  * nested ifs.
 	  */
-	 if_stack.push_tail(cur_if->make_list(mem_ctx));
-	 else_stack.push_tail(cur_else->make_list(mem_ctx));
-	 endif_stack.push_tail(cur_endif->make_list(mem_ctx));
+	 if_stack.push_tail(new(mem_ctx) bblock_link(cur_if));
+	 else_stack.push_tail(new(mem_ctx) bblock_link(cur_else));
 
 	 cur_if = cur;
 	 cur_else = NULL;
-	 /* Set up the block just after the endif.  Don't know when exactly
-	  * it will start, yet.
-	  */
-	 cur_endif = new_block();
+         cur_endif = NULL;
 
 	 /* Set up our immediately following block, full of "then"
 	  * instructions.
@@ -128,40 +127,71 @@ cfg_t::create(void *parent_mem_ctx, exec_list *instructions)
 	 next->start = (backend_instruction *)inst->next;
 	 cur_if->add_successor(mem_ctx, next);
 
-	 set_next_block(next);
+	 set_next_block(&cur, next, ip);
 	 break;
 
       case BRW_OPCODE_ELSE:
-	 cur->add_successor(mem_ctx, cur_endif);
+         cur_else = cur;
 
 	 next = new_block();
 	 next->start = (backend_instruction *)inst->next;
 	 cur_if->add_successor(mem_ctx, next);
-	 cur_else = next;
 
-	 set_next_block(next);
+	 set_next_block(&cur, next, ip);
 	 break;
 
-      case BRW_OPCODE_ENDIF:
-	 cur_endif->start = (backend_instruction *)inst->next;
-	 cur->add_successor(mem_ctx, cur_endif);
-	 set_next_block(cur_endif);
+      case BRW_OPCODE_ENDIF: {
+         if (cur->start == inst) {
+            /* New block was just created; use it. */
+            cur_endif = cur;
+         } else {
+            cur_endif = new_block();
+            cur_endif->start = inst;
 
-	 if (!cur_else)
-	    cur_if->add_successor(mem_ctx, cur_endif);
+            cur->end = (backend_instruction *)inst->prev;
+            cur->add_successor(mem_ctx, cur_endif);
+
+            set_next_block(&cur, cur_endif, ip - 1);
+         }
+
+         backend_instruction *else_inst = NULL;
+         if (cur_else) {
+            else_inst = (backend_instruction *)cur_else->end;
+
+            cur_else->add_successor(mem_ctx, cur_endif);
+         } else {
+            cur_if->add_successor(mem_ctx, cur_endif);
+         }
+
+         assert(cur_if->end->opcode == BRW_OPCODE_IF);
+         assert(!else_inst || else_inst->opcode == BRW_OPCODE_ELSE);
+         assert(inst->opcode == BRW_OPCODE_ENDIF);
+
+         cur_if->if_inst = cur_if->end;
+         cur_if->else_inst = else_inst;
+         cur_if->endif_inst = inst;
+
+	 if (cur_else) {
+            cur_else->if_inst = cur_if->end;
+            cur_else->else_inst = else_inst;
+            cur_else->endif_inst = inst;
+         }
+
+         cur->if_inst = cur_if->end;
+         cur->else_inst = else_inst;
+         cur->endif_inst = inst;
 
 	 /* Pop the stack so we're in the previous if/else/endif */
 	 cur_if = pop_stack(&if_stack);
 	 cur_else = pop_stack(&else_stack);
-	 cur_endif = pop_stack(&endif_stack);
 	 break;
-
+      }
       case BRW_OPCODE_DO:
 	 /* Push our information onto a stack so we can recover from
 	  * nested loops.
 	  */
-	 do_stack.push_tail(cur_do->make_list(mem_ctx));
-	 while_stack.push_tail(cur_while->make_list(mem_ctx));
+	 do_stack.push_tail(new(mem_ctx) bblock_link(cur_do));
+	 while_stack.push_tail(new(mem_ctx) bblock_link(cur_while));
 
 	 /* Set up the block just after the while.  Don't know when exactly
 	  * it will start, yet.
@@ -176,7 +206,7 @@ cfg_t::create(void *parent_mem_ctx, exec_list *instructions)
 	 cur->add_successor(mem_ctx, next);
 	 cur_do = next;
 
-	 set_next_block(next);
+	 set_next_block(&cur, next, ip);
 	 break;
 
       case BRW_OPCODE_CONTINUE:
@@ -187,7 +217,7 @@ cfg_t::create(void *parent_mem_ctx, exec_list *instructions)
 	 if (inst->predicate)
 	    cur->add_successor(mem_ctx, next);
 
-	 set_next_block(next);
+	 set_next_block(&cur, next, ip);
 	 break;
 
       case BRW_OPCODE_BREAK:
@@ -198,14 +228,14 @@ cfg_t::create(void *parent_mem_ctx, exec_list *instructions)
 	 if (inst->predicate)
 	    cur->add_successor(mem_ctx, next);
 
-	 set_next_block(next);
+	 set_next_block(&cur, next, ip);
 	 break;
 
       case BRW_OPCODE_WHILE:
 	 cur_while->start = (backend_instruction *)inst->next;
 
 	 cur->add_successor(mem_ctx, cur_do);
-	 set_next_block(cur_while);
+	 set_next_block(&cur, cur_while, ip);
 
 	 /* Pop the stack so we're in the previous loop */
 	 cur_do = pop_stack(&do_stack);
@@ -236,17 +266,17 @@ cfg_t::new_block()
 }
 
 void
-cfg_t::set_next_block(bblock_t *block)
+cfg_t::set_next_block(bblock_t **cur, bblock_t *block, int ip)
 {
-   if (cur) {
-      assert(cur->end->next == block->start);
-      cur->end_ip = ip - 1;
+   if (*cur) {
+      assert((*cur)->end->next == block->start);
+      (*cur)->end_ip = ip - 1;
    }
 
    block->start_ip = ip;
    block->block_num = num_blocks++;
-   block_list.push_tail(block->make_list(mem_ctx));
-   cur = block;
+   block_list.push_tail(new(mem_ctx) bblock_link(block));
+   *cur = block;
 }
 
 void
@@ -261,3 +291,26 @@ cfg_t::make_block_array()
    }
    assert(i == num_blocks);
 }
+
+void
+cfg_t::dump(backend_visitor *v)
+{
+   for (int b = 0; b < this->num_blocks; b++) {
+        bblock_t *block = this->blocks[b];
+      fprintf(stderr, "START B%d", b);
+      foreach_list(node, &block->parents) {
+         bblock_link *link = (bblock_link *)node;
+         fprintf(stderr, " <-B%d",
+                 link->block->block_num);
+      }
+      fprintf(stderr, "\n");
+      block->dump(v);
+      fprintf(stderr, "END B%d", b);
+      foreach_list(node, &block->children) {
+         bblock_link *link = (bblock_link *)node;
+         fprintf(stderr, " ->B%d",
+                 link->block->block_num);
+      }
+      fprintf(stderr, "\n");
+   }
+}
